import usb.core
import time
from crccheck.crc import Crc32Mpeg2
from .constants import MBC_TYPES, RAM_TYPES, ROM_TYPES
from rich.progress import Progress
import warnings


# Endpoints definitiions for USB Bulk IN and OUT
IN_ENDPOINT = 0x81
OUT_ENDPOINT = 0x01

# GB Operator Vendor and Product IDs (firmware 9.0+)
GB_OPERATOR_VENDOR_ID = 0x16d0
GB_OPERATOR_PRODUCT_ID = 0x123d

# Old GB Operator Vendor and Product IDs ( firmware pre 9.0)

OLD_GB_OPERATOR_VENDOR_ID = 0x1D50
OLD_GB_OPERATOR_PRODUCT_ID = 0x6018

# Trigger sequence
TRIGGER_CARTRIDGE_INFO = bytearray(
    [
        0x04,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
    ]
)
TRIGGER_SAVE_READ = bytearray(
    [
        0x02,
        0x01,
        0x00,
        0x00,
        0x00,
        0x01,
        0x00,
        0x02,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
        0x00,
    ]
)


def _craft_save_write_trigger(save_size):
    """
    Craft a save write trigger

    Parameters
    ----------
    save_size : int

    Returns
    -------
    bytearray
    """

    save_size_bytearray = save_size.to_bytes(
        (save_size.bit_length() + 7) // 8, byteorder="little"
    )
    trigger_save_write = bytearray([0x03, 0x00, 0x00, 0x00, 0x00, 0x00])
    trigger_save_write += save_size_bytearray

    # Add 0x00 to reach 60 bytes
    pad_legnth = 60 - len(trigger_save_write)
    trigger_save_write += bytearray([0x00] * pad_legnth)
    # Add CRC32
    trigger_save_write = add_crc32(trigger_save_write)
    return trigger_save_write


def _craft_rom_read_trigger(rom_size):
    """
    Craft a rom read trigger

    Parameters
    ----------
    rom_size : int

    Returns
    -------
    bytearray
    """

    rom_size_bytearray = rom_size.to_bytes(
        (rom_size.bit_length() + 7) // 8, byteorder="little"
    )
    trigger_rom_read = bytearray([0x00, 0x00])
    trigger_rom_read += rom_size_bytearray

    # Add 0x00 to reach 60 bytes
    pad_legnth = 60 - len(trigger_rom_read)
    trigger_rom_read += bytearray([0x00] * pad_legnth)
    # Add CRC32
    trigger_rom_read = add_crc32(trigger_rom_read)
    return trigger_rom_read


def _craft_save_read_trigger(save_size):
    """
    Craft a save read trigger

    Parameters
    ----------
    save_size : int

    Returns
    -------
    bytearray
    """

    save_size_bytearray = save_size.to_bytes(
        (save_size.bit_length() + 7) // 8, byteorder="little"
    )
    trigger_save_read = bytearray([0x02, 0x00, 0x00, 0x00, 0x00])
    trigger_save_read += save_size_bytearray

    # Add 0x00 to reach 60 bytes
    pad_legnth = 60 - len(trigger_save_read)
    trigger_save_read += bytearray([0x00] * pad_legnth)
    # Add CRC32
    trigger_save_read = add_crc32(trigger_save_read)
    return trigger_save_read


def add_crc32(data):
    """
    Add CRC32 to data

    Parameters
    ----------
    data : bytearray

    Returns
    -------
    bytearray
    """
    crc = Crc32Mpeg2.calc(data)
    bytes_crc = crc.to_bytes((crc.bit_length() + 7) // 8, byteorder="little")
    return data + bytes_crc


def find_gb_operator():
    """
    Find GB Operator device

    Returns
    -------
    usb.core.Device
    """
    # Find GB Operator with vendor ID and product ID
    # Those values can be found with `lsusb`
    
    # Find device with firmware v9.0+
    found_device = usb.core.find(
        idVendor=GB_OPERATOR_VENDOR_ID, idProduct=GB_OPERATOR_PRODUCT_ID
    )
    # Find older devices 
    found_old_device = usb.core.find(
            idVendor=OLD_GB_OPERATOR_VENDOR_ID, idProduct=OLD_GB_OPERATOR_PRODUCT_ID
    )


    if (found_device is not None) and (found_old_device is not None):
        warnings.warn("Found 2 GB Operators, one of which with and old firmware. \
                      GB Opyrator will use the one with the newer firmware", category=Warning)
        return found_device
    elif found_device is not None:
        return found_device
    elif found_old_device is not None:
        warnings.warn("Found GB Operator with old firmware. Please update it with Playback from epilogue.co")
        return found_old_device



def find_gb_operator_blocking(timeout=0):
    """
    Block while GB Operator device if not found

    Parameters
    ----------
    timeout : int, optional. If timeout = 0, will nerver timeout

    Returns
    -------
    usb.core.Device
    """

    if timeout == 0:
        while True:
            gbop_device = find_gb_operator()
            if gbop_device is not None:
                return gbop_device
            time.sleep(1)
    else:
        start = time.time()
        while True:
            gbop_device = find_gb_operator()
            if gbop_device is not None:
                return gbop_device
            time.sleep(1)
            if time.time() - start > timeout:
                raise TimeoutError("Unable to find GB Operator device")


def init_gb_operator(gbop_device):
    """
    Initialize GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device
    """

    # Check if device is None
    if gbop_device is None:
        raise ValueError("Device not found, you passed a device that is None")

    # Detack kernel driver
    if gbop_device.is_kernel_driver_active(0):
        gbop_device.detach_kernel_driver(0)

    # Activate first configuration
    gbop_device.set_configuration()

    # return device
    return gbop_device


def read_bulk_in(gbop_device, num_bytes=0, with_ack=False, quiet=False):
    """
    Read data from GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device
    num_bytes : int, optional

    Returns
    -------
    bytearray
    """
    received_data = bytearray([])

    # Read data until GB Operator stops responding
    iteration = 0

    with Progress(disable=quiet, transient=True) as progress:
        task = progress.add_task("Reading...", total=num_bytes)

        while len(received_data) < num_bytes:
            progress.update(task, advance=64)
            try:
                usbpacket_data = gbop_device.read(IN_ENDPOINT, 64)
                received_data += bytearray(usbpacket_data)
                iteration += 1
            except usb.core.USBTimeoutError:
                print("USBTimeoutError")
                return received_data
            if with_ack and (iteration % 320 == 0):
                # send ACK
                gbop_device.write(OUT_ENDPOINT, bytearray([0x00] * 64))
                # read ACK
                _ = gbop_device.read(IN_ENDPOINT, 60)
                _ = gbop_device.read(IN_ENDPOINT, 4)

    return received_data


def write_bulk_out(gbop_device, data, qiuet=False):
    """
    Write data to GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device
    data : bytearray
    """
    with Progress(disable=qiuet, transient=True) as progress:
        task = progress.add_task("Writing...", total=len(data))

        # Write data to GB Operator in chunks of 64 bytes
        for sequence in range(0, len(data), 64):
            progress.update(task, advance=64)
            gbop_device.write(OUT_ENDPOINT, data[sequence : sequence + 64])
            time.sleep(0.0001)
            # Read 64 bytes from GB Operator
            _ = gbop_device.read(IN_ENDPOINT, 60)
            _ = gbop_device.read(IN_ENDPOINT, 4)


def read_cartridge_info(gbop_device):
    """
    Read cartridge info from GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device

    Returns
    -------
    dict
    """
    # Send trigger_bytes
    gbop_device.write(OUT_ENDPOINT, add_crc32(TRIGGER_CARTRIDGE_INFO))

    # Burn the ACK
    _ = gbop_device.read(IN_ENDPOINT, 60)
    _ = gbop_device.read(IN_ENDPOINT, 4)

    # Read card data from GB Operator
    received_data = read_bulk_in(gbop_device, num_bytes=256, quiet=True)

    # Extract card info only
    received_data = received_data[:60]

    # check if received_data is all null bytes
    if not (received_data[3] or received_data[4]):
        return None

    if received_data[2] == 0x20:
        # Parse data
        cartridge_info = {
            "cartridge_type": "GB/GBC",
            "ROM_size": int.from_bytes(received_data[5:8], byteorder="little"),
            "RAM_size": int.from_bytes(received_data[9:12], byteorder="little"),
            "title_first_letter": chr(received_data[13]),
            "MBC_type": MBC_TYPES[received_data[14]],
            "ROM_type": ROM_TYPES[received_data[15]],
            "RAM_type": RAM_TYPES[received_data[16]],
            "heasder_checksum": received_data[17],
            "global_checksum": received_data[18:20],
        }
    else:
        cartridge_info = {
            "cartridge_type": "GBA",
            # "ROM_size": int.from_bytes(received_data[5:8], byteorder="little"),
            # "RAM_size": int.from_bytes(received_data[9:12], byteorder="little"),
        }
        raise NotImplementedError("GBA cartridge support not implemented yet")

    return cartridge_info


def read_save(gbop_device, num_bytes, quiet=False):
    """
    Dump save file from GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device
    filename : str
    """
    # craft trigger bytes
    trigger_save_read = _craft_save_read_trigger(num_bytes)

    # Send trigger_bytes
    gbop_device.write(OUT_ENDPOINT, trigger_save_read)

    # Burn the first 64 bytes
    # Burn first 64 bytes in 2 times
    _ = gbop_device.read(IN_ENDPOINT, 60)
    _ = gbop_device.read(IN_ENDPOINT, 4)

    # Read data from GB Operator
    received_data = read_bulk_in(gbop_device, num_bytes=num_bytes, quiet=quiet)

    return bytearray(received_data)


def write_save(gbop_device, bytearray_data, quiet=False):
    """
    Write save file to GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device
    filename : str
    """
    # craft trigger bytes
    trigger_save_write = _craft_save_write_trigger(len(bytearray_data))

    # Send trigger_bytes
    gbop_device.write(OUT_ENDPOINT, trigger_save_write)

    # Burn the first 64 bytes
    # Burn first 64 bytes in 2 times
    _ = gbop_device.read(IN_ENDPOINT, 60)
    _ = gbop_device.read(IN_ENDPOINT, 4)

    # Write data to GB Operator
    write_bulk_out(gbop_device, bytearray_data, qiuet=quiet)

    return True


def read_rom(gbop_device, num_bytes, quiet=False):
    """
    Dump ROM from GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device
    filename : str
    """
    # craft trigger bytes
    trigger_rom_read = _craft_rom_read_trigger(num_bytes)

    # Send trigger_bytes
    gbop_device.write(OUT_ENDPOINT, trigger_rom_read)

    # Burn first 64 bytes in 2 times
    _ = gbop_device.read(IN_ENDPOINT, 60)
    _ = gbop_device.read(IN_ENDPOINT, 4)

    # Send "ACK" to GB Operator
    gbop_device.write(OUT_ENDPOINT, bytearray([0x00] * 64))

    # Burn first 64 bytes a second time
    _ = gbop_device.read(IN_ENDPOINT, 60)
    _ = gbop_device.read(IN_ENDPOINT, 4)

    # Read data from GB Operator
    received_data = read_bulk_in(
        gbop_device, num_bytes=num_bytes, with_ack=True, quiet=quiet
    )

    return bytearray(received_data)


def release_gb_operator(gbop_device):
    """
    Release GB Operator device

    Parameters
    ----------
    gbop_device : usb.core.Device
    """
    # Release device
    usb.util.dispose_resources(gbop_device)



import binascii
import time
import re
from rich.console import Console
from . import coms_utils as cu
from .printer import Printer
import sys

def check_initialized(func):
    def wrapper(*args, **kwargs):
        if args[0].initialized:
            return func(*args, **kwargs)
        else:
            raise Exception(
                "Reader not initialized, call initialize_reader() or initialize_reader_blocking() first"
            )

    return wrapper


def release_device(func):
    def wrapper(*args, **kwargs):
        output = func(*args, **kwargs)
        cu.release_gb_operator(args[0].gbop_device)
        return output

    return wrapper


def get_cartridge_info(func):
    def wrapper(*args, **kwargs):
        cartridge_info = cu.read_cartridge_info(args[0].gbop_device)
        if cartridge_info is None:
            raise Exception(
                "Could not read cartridge info. Make sure a cartridge is inserted."
            )

        return func(*args, cartridge_info=cartridge_info, **kwargs)

    return wrapper


class CartridgeReader(object):
    def __init__(self, quiet=False):
        self.initialized = False
        self.console = Console()
        self.quiet = quiet
        self.printer = Printer(quiet=quiet)

    def initialize_reader(self, blocking=False, timeout=0):
        if not blocking and timeout != 0:
            print("[WARNING] timeout is ignored when blocking is set to Fale",file=sys.stderr)

        with self.printer.status("Initializing reader..."):
            if blocking:
                dev = cu.find_gb_operator_blocking(timeout=timeout)
            else:
                dev = cu.find_gb_operator()

        if dev is None:
            raise ValueError(
                """
                GB Operator not found, check it is plugged in or that the appropriate udev rules are set up if you are on linux. See : https://support.epilogue.co/hc/en-us/articles/4403827118738-How-can-I-connect-my-Operator-device-on-Linux-under-a-non-root-user-
                """
            )
        else:
            self.gbop_device = cu.init_gb_operator(dev)
            self.initialized = True
            self.printer.success("[bold green]Reader initialized[/bold green]")

        return

    @check_initialized
    @release_device
    def read_cartridge_info(self):
        # with self.printer.status("Reading cartridge info..."):
        _cartridge_info = cu.read_cartridge_info(self.gbop_device)
        return _cartridge_info

    @check_initialized
    @release_device
    @get_cartridge_info
    def read_rom(self, cartridge_info=None):
        # with self.printer.status("Reading ROM..."):
        num_bytes = cartridge_info["ROM_size"]
        return cu.read_rom(self.gbop_device, num_bytes, quiet=self.quiet)

    def dump_rom(self, filename):
        rom = self.read_rom()
        with open(filename, "wb") as f:
            f.write(rom)
        self.printer.success(f"ROM dumped to:\t[dark_cyan]{filename}[/dark_cyan]")

    @check_initialized
    @release_device
    @get_cartridge_info
    def read_save(self, cartridge_info=None):
        num_bytes = cartridge_info["RAM_size"]
        if num_bytes == 0:
            self.printer.error("No RAM (save) detected on this cartridge.")
            return None
        else:
            # with self.printer.status("Reading save..."):
            return cu.read_save(self.gbop_device, num_bytes, quiet=self.quiet)

    def dump_save(self, filename):
        save = self.read_save()
        if save is not None:
            with open(filename, "wb") as f:
                f.write(save)
            self.printer.success(f"Save dumped to:\t[dark_cyan]{filename}[/dark_cyan]")

    @check_initialized
    @release_device
    @get_cartridge_info
    def write_save(self, data, cartridge_info=None):
        num_bytes = cartridge_info["RAM_size"]
        if num_bytes == 0:
            self.printer.error(
                "No RAM (save) detected on this cartridge. Impossible to write save."
            )
            return None
        elif num_bytes != len(data):
            self.printer.error(
                f"Save size mismatch. Expected {num_bytes} bytes, got {len(data)} bytes."
            )
            return None
        # with self.printer.status("Writing save..."):
        return cu.write_save(self.gbop_device, data, quiet=self.quiet)

    def write_save_from_file(self, filename):
        with open(filename, "rb") as f:
            data = f.read()
        time.sleep(1)
        out = self.write_save(data)
        if out is not None:
            self.printer.success(
                f"Save written from:\t[dark_cyan]{filename}[/dark_cyan]"
            )

    def close(self):
        self.gbop_device.attach_kernel_driver(0)

    @check_initialized
    @get_cartridge_info
    def get_epilogue_id(self, cartridge_info=None):
        epilogue_id = (
            cartridge_info["title_first_letter"].upper()
            + "{:02x}".format(cartridge_info["heasder_checksum"]).upper()
            + binascii.hexlify(cartridge_info["global_checksum"]).decode().upper()
        )
        return epilogue_id


def file_crc32(filename):
    with open(filename, "rb") as f:
        data = f.read()
    out = binascii.crc32(data) & 0xFFFFFFFF
    return hex(out)


def bytearray_crc32(data):
    out = binascii.crc32(data) & 0xFFFFFFFF
    return hex(out)


def create_crc_db(filename):
    with open(filename, "r") as f:
        content = f.read()
    game_regex = re.compile(
        r'game \(\n\tcomment "(.*)"\n\tpublisher "(.*)"\n\trom \( crc (.*) \)\n\)'
    )
    games = game_regex.findall(content)
    db = {}
    for game in games:
        db[game[2]] = game
    return db



# %%
import argparse
from .cartridge_utils import CartridgeReader
from gbopyrator import load_roms_db

# %%
def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--dump-rom", type=str, default=None, help="Dump ROM to file")
    parser.add_argument("--dump-save", type=str, default=None, help="Dump save to file")
    parser.add_argument(
        "--write-save", type=str, default=None, help="Write save from file"
    )
    parser.add_argument(
        "--quiet",
        action="store_true",
        default=False,
        help="Do not output anything to stdout",
    )
    args = parser.parse_args()

    # %%

    cr = CartridgeReader(quiet=args.quiet)

    cr.printer.greetings()

    if (args.dump_save is not None) and (args.write_save is not None):
        cr.printer.warning(
            "`dump-save` and `write-save` are both set. GBOpyrator will dump the save first and write it after."
        )
    cr.initialize_reader(blocking=True,timeout=10)

    rom_epilogue_id = cr.get_epilogue_id()
    roms_db = load_roms_db()

    # Print cartridge info
    if rom_epilogue_id in roms_db:
        rom_info = roms_db[rom_epilogue_id]
        # Center "rom info" text on =80 chars
        cr.printer.print("")
        cr.printer.rule("[blue_violet]CARTRIDGE INFO")
        cr.printer.print(
            f"""Detected game:\t[blue_violet]{rom_info['full_title']}[/blue_violet]"""
        )
        if rom_info["SGB_support"]:
            cr.printer.print(f"""SGB support:\t[blue_violet]Yes[/blue_violet]""")
        if rom_info["CGB_support"]:
            cr.printer.print(f"""CGB support:\t[blue_violet]Yes[/blue_violet]""")
        cr.printer.print(
            f"""ROM size:\t[blue_violet]{rom_info['ROM_size']}[/blue_violet]"""
        )
        if rom_info["RAM_size"] != 0:
            cr.printer.print(
                f"""RAM size:\t[blue_violet]{rom_info['RAM_size']}[/blue_violet]"""
            )
    else:
        cr.printer.warning(
            f"ROM epilogue ID not found in the database. If you know the game, please add it to the database."
        )

    # Print dumping/writing info
    if (
        args.dump_save is not None
        or args.dump_rom is not None
        or args.write_save is not None
    ):
        cr.printer.print("")
        cr.printer.rule("[blue_violet]ROM AND SAVE OPERATIONS")

        if args.dump_save is not None:
            cr.dump_save(args.dump_save)

        if args.dump_rom is not None:
            cr.dump_rom(args.dump_rom)

        if args.write_save is not None:
            cr.write_save_from_file(args.write_save)

        cr.close()

    cr.printer.print("")


# %%

if __name__ == "__main__":
    main()



from rich.console import Console


class Printer(Console):
    def __init__(self, quiet=False):
        super().__init__(width=80)
        self.quiet = quiet

    def print(self, *args, **kwargs):
        if not self.quiet:
            super().print(*args, **kwargs)

    def greetings(self):
        if not self.quiet:
            self.print(
                """[bold blue_violet]
            __________     ____                         __            
           / ____/ __ )   / __ \____  __  ___________ _/ /_____  _____
          / / __/ __  |  / / / / __ \/ / / / ___/ __ `/ __/ __ \/ ___/
         / /_/ / /_/ /  / /_/ / /_/ / /_/ / /  / /_/ / /_/ /_/ / /    
         \____/_____/   \____/ .___/\__, /_/   \__,_/\__/\____/_/     
                            /_/    /____/                             
            [/bold blue_violet]"""
            )

    def warning(self, message):
        if not self.quiet:
            self.print(f"[bold orange1]⚠️ {message}[/bold orange1]")

    def error(self, message):
        if not self.quiet:
            self.print(f"[bold red]❌ {message}[/bold red]")

    def success(self, message):
        if not self.quiet:
            self.print(f"[bold green]✅ {message}[/bold green]")

    def status(self, *args, **kwargs):
        if not self.quiet:
            return super().status(*args, **kwargs)
        else:
            return EmptyContextManager()


class EmptyContextManager:
    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_value, traceback):
        pass



from setuptools import setup, find_packages


def read_requirements(file):
    with open(file, "r") as f:
        return f.read().splitlines()


setup(
    name="gbopyrator",
    version="0.5",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "gbopyrator = gbopyrator.gbopyrator:main",
        ],
    },
    package_data={"gbopyrator": ["gb_gbc_roms_info.json"]},
    install_requires=read_requirements("requirements.txt"),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    python_requires=">=3.7",
    url="https://github.com/N0ciple/gbopyrator",
)



MBC_TYPES = {
    0x00: "ROM ONLY",
    0x01: "MBC1",
    0x02: "MBC1+RAM",
    0x03: "MBC1+RAM+BATTERY",
    0x05: "MBC2",
    0x06: "MBC2+BATTERY",
    0x08: "ROM+RAM 1",
    0x09: "ROM+RAM+BATTERY 1",
    0x0B: "MMM01",
    0x0C: "MMM01+RAM",
    0x0D: "MMM01+RAM+BATTERY",
    0x0F: "MBC3+TIMER+BATTERY",
    0x10: "MBC3+TIMER+RAM+BATTERY 2",
    0x11: "MBC3",
    0x12: "MBC3+RAM 2",
    0x13: "MBC3+RAM+BATTERY 2",
    0x19: "MBC5",
    0x1A: "MBC5+RAM",
    0x1B: "MBC5+RAM+BATTERY",
    0x1C: "MBC5+RUMBLE",
    0x1D: "MBC5+RUMBLE+RAM",
    0x1E: "MBC5+RUMBLE+RAM+BATTERY",
    0x20: "MBC6",
    0x22: "MBC7+SENSOR+RUMBLE+RAM+BATTERY",
    0xFC: "POCKET CAMERA",
    0xFD: "BANDAI TAMA5",
    0xFE: "HuC3",
    0xFF: "HuC1+RAM+BATTERY",
}

ROM_TYPES = {
    0x00: {"ROM_size_info": "32 KiB", "num_rom_banks": 2},
    0x01: {"ROM_size_info": "64 KiB", "num_rom_banks": 4},
    0x02: {"ROM_size_info": "128 KiB", "num_rom_banks": 8},
    0x03: {"ROM_size_info": "256 KiB", "num_rom_banks": 16},
    0x04: {"ROM_size_info": "512 KiB", "num_rom_banks": 32},
    0x05: {"ROM_size_info": "1 MiB", "num_rom_banks": 64},
    0x06: {"ROM_size_info": "2 MiB", "num_rom_banks": 128},
    0x07: {"ROM_size_info": "4 MiB", "num_rom_banks": 256},
    0x08: {"ROM_size_info": "8 MiB", "num_rom_banks": 512},
}

RAM_TYPES = {
    0x00: {"SRAM_size_info": "0", "info": "No RAM"},
    0x02: {"SRAM_size_info": "8 KiB", "info": "1 bank"},
    0x03: {"SRAM_size_info": "32 KiB", "info": "4 banks of 8 KiB each"},
    0x04: {"SRAM_size_info": "128 KiB", "info": "16 banks of 8 KiB each"},
    0x05: {"SRAM_size_info": "64 KiB", "info": "8 banks of 8 KiB each"},
}



from .cartridge_utils import CartridgeReader
from pkg_resources import resource_filename
import json

__version__ = "0.5"

def load_roms_db(filename=resource_filename(__name__, "gb_gbc_roms_info.json")):
    with open(filename, "r") as file:
        roms_db = json.load(file)
    return roms_db
